#ifndef __GTINFO_H_
#define __GTINFO_H_

#include <sys/types.h>

#include <DAQDataFormats/RawTrigger.h>
#include <DAQDataFormats/TriggerDefines.h>

#include <NovaGlobalTrigger/GT.h>
#include <NovaGlobalTrigger/GT_Throttle.h>

GT_NAMESPACE_BEGIN

class GTInfo {

public:

  GTInfo();   // Constructor
  // ~GTInfo();   // Destructor not needed?

  // methods

  uint32_t getLength();                    // get the trigger length
  void     setLength(uint32_t Length);     // set the trigger length

  uint32_t getPrescale();                  // get the trigger prescale
  void     setPrescale(uint32_t Prescale); // set the trigger prescale

  bool     getEnabled();                   // is this trigger enabled?
  void     setEnabled(bool Enabled);       // enable/disable it

  int64_t  getDelay();                     // get the trigger delay
  void     setDelay(int64_t Delay);        // set the trigger delay

  std::string   getName();                 // get the trigger name
  void     setName(std::string Name);      // set the trigger name

  uint64_t getSeenCount();                 // get the trigger count
  uint64_t incSeenCount();                 // increment the trigger count
  void     resetSeenCount();               // clear the trigger count

  uint64_t getSentCount();                 // get the trigger count
  uint64_t incSentCount();                 // increment the trigger count
  void     resetSentCount();               // clear the trigger count

  uint64_t getMissedCount();               // get the trigger count
  uint64_t incMissedCount();               // increment the trigger count
  void     resetMissedCount();             // clear the trigger count

  daqdataformats::trigID   getType();      // get the trigger mask
  void     setType(daqdataformats::trigID);// set the trigger mask

  bool     getRandom();                    // get the random flag
  void     setRandom(bool);                // set the random flag

  float    getRate();                      // get the trigger rate
  void     setRate(float);                 // set the trigger rate

  bool     getDDT();                       // get the trigger rate
  void     setDDT(bool);                   // set the trigger rate

  void     initMetric();                   // set up ganglia rate
  void     logDDT(uint32_t, uint32_t, uint32_t); // fill DDT ganglia metrics

  throttlePipeline* Queue;                 // The throttling Q for this trig

private:

  // data
  uint32_t Length;
  uint32_t Prescale;
  bool     Enabled;
  int64_t  Delay;
  std::string   Name;   // The configured trigger name, not the bitset name
  uint64_t SeenCount;   // triggers seen before prescale
  uint64_t SentCount;   // triggers actually sent
  uint64_t MissedCount; // triggers lost due to throttling
  daqdataformats::trigID   Type;
  int32_t  TriggerRate; // ganglia metric for trigger rate
  int32_t  DDTLength;   // ganglia metric for DDT trigger length
  int32_t  DDTDecision; // ganglia metric for DDT trigger time to decision
  int32_t  DDTDelay;    // ganglia metric for DDT trigger delay
  bool     DDT;         // is this a data-driven trigger?
  bool     Random;      // is this a random (vs periodic) trigger?
  float    Rate;        // the trigger rate (for self-triggered triggers)

  // methods

};

GT_NAMESPACE_END

#endif // __GTINFO_H_
