#ifndef __GT_INST_H_
#define __GT_INST_H_

#include <sys/types.h>
#include <sys/wait.h>
#include <boost/thread/mutex.hpp>
#include <string.h>

#include <NovaGlobalTrigger/GT.h>

GT_NAMESPACE_BEGIN

// the class definition cannot be extern
// so it must be put in there manually when the instance is declared
// in the non-instantiating files
class GT_Inst {

public:

  // set this up for Singleton-ness
  static GT_Inst& getInstance();

  // methods
  int32_t spawnGT(int32_t partno);       // fire off a GT on partition
  int32_t killGT(int32_t partno);        // kill a GT on partition
  bool setConfigURI(std::string URI, int32_t partno); // set the config file
  std::string getConfigURI(int32_t partno);
  int32_t ClearPID (pid_t pid);        // find partition on which pid
                                       // was running and reset it
                                       // This is public so the signal 
                                       // handler can call it when a 
                                       // partition dies

private:

  explicit GT_Inst();   // Constructor
  //      ~GT_Inst();   // Destructor not needed?

  // disallow any copying
  GT_Inst(GT_Inst const&);  // not implemented
  void operator= (GT_Inst const&);  // not implemented

  // data
  pid_t GTProcs[NUMPARTS]; 
  std::string configURI[NUMPARTS];
  boost::mutex GTProcs_mutex;

  // methods
  bool inrange(int32_t partno) const;
  int32_t GetPID (int32_t partno) const; // return pid of GT on partition
  boost::mutex GT_Inst_mutex;
};

GT_NAMESPACE_END

#endif // __GT_INST_H_
