#ifndef __GLOBAL_TRIGGER_SEND_QUEUES_H
#define __GLOBAL_TRIGGER_SEND_QUEUES_H

#include <string>
#include <map>
#include <deque>

#include <boost/thread/mutex.hpp>

#include <DAQMessages/ccpp_GTMessages.h>

#include <NovaGlobalTrigger/GT.h>
#include <NovaGlobalTrigger/GTInfo.h>
#include <NovaGlobalTrigger/GT_MessageQueue.h>

GT_NAMESPACE_BEGIN

class GT_SendQueues {

 public:

  static GT_SendQueues& getInstance();

  bool QueueTrigger(std::string QueueName, triggermessages::TriggerMessage& m);
  // triggermessages::TriggerMessage& GetNextTrigger();
  triggermessages::TriggerMessage* GetNextTrigger();


  bool AddQueue(std::string       QueueName, int priority);
  bool RemoveQueue(std::string    QueueName);
  void ChangePriority(std::string QueueName, int priority);
  int  GetPriority(std::string    QueueName);
  void ListQueues();
  void setDebug(bool state){_debug=state;};
  void setPollValue(int t){_pollVal=t;};
  void setPollTimeout(int t){_pollTimeOut=t;};
  bool getDebug(){return _debug;};
  int getPollValue(){return _pollVal;};
  int getPollTimeout(){return _pollTimeOut;};

  enum QueueExceptions {
    ExceptionOffset = 100,
    NextTrigger_QueuesEmpty,
    NextTrigger_TimeOut
  };

 private:

  explicit GT_SendQueues():_debug(false),_pollTimeOut(-1){};                                // Private Constructor

  bool _debug;                                               // Turn debugging on or off
  int  _pollVal;                                             // Polling value (microseconds)
  int  _pollTimeOut;                                         // Timeout value for queue polling

  std::map<std::string,messageQueue*>  theQueueMap;          // Map to the messageQueues
  std::map<std::string,int>            theQueuePriority;     // Map to the messageQueue priorities
  std::map<int,messageQueue*>          theAccessQueues;      // Prioritized queues for read access
  std::map<std::string,boost::mutex>   QueueAccess_mutex;    // Map to the mutexes for queue access


}; // endclass GT_SendQueues

GT_NAMESPACE_END



#endif /* __GLOBAL_TRIGGER_SEND_QUEUES_H */
