#ifndef _NGT_MASTER_RC_H
#define _NGT_MASTER_RC_H

#include <NovaDAQUtilities/Runnable.h>
#include <NovaRunControlClient/RunControlReceiver.h>
#include <DAQMessages/ccpp_RunControlMessages.h>
#include <messagefacility/MessageLogger/MessageLogger.h>

#include <NovaGlobalTrigger/NGTMCStateMachine.h>

#include <boost/shared_ptr.hpp>
#include <boost/thread/mutex.hpp>

#include <NovaGlobalTrigger/GT.h>

GT_NAMESPACE_BEGIN

class NGTMasterRC : public novadaq::Runnable, public NGTMCStateMachine {

public:

  /**
   * Creates an instance with the specified application name.
   */
  NGTMasterRC(std::string applicationName, int partition);

  /**
   * Destructor to clean up any allocated memory.
   */
  ~NGTMasterRC();

  /**
   * Contains the operations that are executed in a separate thread.
   */
  void run (); 

  // RC client methods:

  /**
   * Handles load run configuration messages.
   */
  boost::shared_ptr<runcontrolmessages::LoadRunConfigurationResponse>
    handleMessage(boost::shared_ptr<runcontrolmessages::LoadRunConfigurationRequest> message);

  /**
   * Handles configure run messages.
   */
  boost::shared_ptr<runcontrolmessages::ConfigureRunResponse>
    handleMessage(boost::shared_ptr<runcontrolmessages::ConfigureRunRequest>
	          message);
  /**
   * Handles heartbeat messages.
   */
  boost::shared_ptr<runcontrolmessages::StatusResponse>
    handleMessage(boost::shared_ptr<runcontrolmessages::StatusRequest> message);

  // accessor methods
  std::string getApplicationName() const { return _applicationName; }

private:
  // variables
  std::string _applicationName;
  boost::shared_ptr<rcclient::RunControlReceiver> _rcReceiver;
  boost::shared_ptr<mf::Presence> _mfPresence;
  mutable boost::mutex _callbackMutex;

  // methods which do stuff upon state changes
  bool _establishPartition(int32_t partition);
  bool _loadRunConfiguration ( std::string configURI, int32_t partition);
  bool _configureRun(int32_t partition);
  bool _deconfigureRun(int32_t partition);
  void _unloadRunConfiguration(int32_t partition);
  bool _detachFromPartition(int32_t partition);
};

GT_NAMESPACE_END

#endif
