#ifndef __NGT_STATEMACHINE_H_
#define __NGT_STATEMACHINE_H_

#include <NovaGlobalTrigger/GT.h>

GT_NAMESPACE_BEGIN

/**
 * NGTMCStateMachine
 *
 * This class implements a simple state machine to manage transitions 
 * between NGT States.
 *
 * Stolen from Sue's DcmStateMachine
 */

class NGTStateMachine {

public:

  // enums and typedefs
  typedef enum NGTState {
    RUN_CONFIGURED,
    RUNNING,
    RUN_PAUSED
  } NGTState_t;

  // static methods
  static std::string NGTStateAsString(NGTState_t state);

  // constructors/destructor
  NGTStateMachine ();
  virtual ~NGTStateMachine ();

  // state changing methods
  bool changeState(NGTState_t state);

  // accessor methods
  NGTState_t getCurrentState() const { return _currentState; }

private:

  NGTState_t _currentState; // current state

  // Private pure virtual methods
  virtual bool _beginRun () = 0;
  virtual bool _pauseRun () = 0;
  virtual void _resumeRun () = 0;
  virtual void _stopRun () = 0;

  // Private methods used by changeState
  bool _changeStateFromRunConfigured(NGTState_t state);
  bool _changeStateFromRunning(NGTState_t state);
  bool _changeStateFromRunPaused(NGTState_t state);

};

GT_NAMESPACE_END

#endif
