/*
 * SNEWSXmlRpcServerClasses header file 
 */
#ifndef __SNEWS_XMLRPCSERVERCLASSES_H_
#define __SNEWS_XMLRPCSERVERCLASSES_H_

/**
 * Class SNEWSXmlRpcServerClasses
 *
 * These classes define the XmlRpc methods that are used to send
 * information from near to far.
 * Adapted for SNEWS messages by J. Zirnstein (janzirn@fnal.gov)
 * 
 * @author H. Meyer
 * @author <a href="mailto:holger.meyer@wichita.edu">holger.meyer@wichita.edu</a>
 */

#include "XmlRpc/XmlRpc.h"
#include "rms/provider/DDSConnection.h"
#include "rms/base/RmsDestination.h"
#include "rms/RmsReceiver.h"
#include "rms/RmsSender.h"
#include "DAQMessages/ccpp_SNEWSMessages.h"
#include <string>
#include <vector>
#include <list>

namespace nova {
namespace NovaSNEWSInterface {

class SNEWSMessage : public XmlRpc::XmlRpcServerMethod
{
 public:
  SNEWSMessage(XmlRpc::XmlRpcServer* s, boost::shared_ptr<gov::fnal::cd::rms::RmsSender<gov::fnal::cd::rms::provider::DDSConnection,nsimessages::SNEWSMessage> > p);

  void execute(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);

 private:
  boost::shared_ptr<gov::fnal::cd::rms::RmsSender<gov::fnal::cd::rms::provider::DDSConnection,
                    nsimessages::SNEWSMessage> >
        requestSenderptr;
};

class Dest
{
 public:
  Dest(std::string const& h,
       int const& p,
       std::list<int> const& tlist);

  std::string host;
  int port;
  std::list < int > typelist;
  XmlRpc::XmlRpcClient client;
};

class DestList //singleton, only one destination list per application
{
 public:
  static DestList& Instance();
  ~DestList() {;}

  std::vector< Dest > const& ReceiverList() {return recList;}

  int addDestination(std::string const & h,
		     int const & p,
		     std::list<int> const& tlist);

  void addAvailableMessageType(int const& t);
  std::list< int > const& AvailableMessageTypes() {return availableMessageTypes;}

 private:
  DestList();
  static DestList* fInstance;
  std::vector< Dest > recList;
  std::list< int > availableMessageTypes;

  void addList(std::vector<std::pair<std::basic_string<char>, int> > const& rL);
};

class SNEWSMessageForward : public XmlRpc::XmlRpcServerMethod
{
 public:
  SNEWSMessageForward(XmlRpc::XmlRpcServer* s);

  void execute(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);
  void DispatchTheSNEWSMessage(XmlRpc::XmlRpcValue& params, int i );


 private:
};

class DestinationAdd : public XmlRpc::XmlRpcServerMethod
{
 public:
  DestinationAdd(XmlRpc::XmlRpcServer* s);

  void execute(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);

};

class DestinationRemove : public XmlRpc::XmlRpcServerMethod
{
 public:
  DestinationRemove(XmlRpc::XmlRpcServer* s);

  void execute(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);

};

} // end of namespace NovaSNEWSInterface
} // end of namespace nova

#endif //__SNEWS_XMLRPCSERVERCLASSES_H_
