/**
 * NssXmlRpcServerClasses header file 
 * $Id: NssXmlRpcServerClasses.h,v 1.7 2013/09/26 15:54:13 edniner Exp $
 */
#ifndef __NSS_XMLRPCSERVERCLASSES_H_
#define __NSS_XMLRPCSERVERCLASSES_H_

/**
 * Class NssXmlRpcServerClasses
 *
 * These classes define the XmlRpc methods that are used to send
 * information from near to far.
 * 
 * @author H. Meyer
 * @author <a href="mailto:holger.meyer@wichita.edu">holger.meyer@wichita.edu</a>
 */

#include "XmlRpc/XmlRpc.h"
#include "rms/provider/DDSConnection.h"
#include "rms/base/RmsDestination.h"
#include "rms/RmsReceiver.h"
#include "rms/RmsSender.h"
#include "DAQMessages/ccpp_NssMessages.h"
#include <string>
#include <vector>
#include <list>

namespace nova {
namespace NovaSpillServer {

class Spill : public XmlRpc::XmlRpcServerMethod
{
 public:
  Spill(XmlRpc::XmlRpcServer* s, boost::shared_ptr<gov::fnal::cd::rms::RmsSender<gov::fnal::cd::rms::provider::DDSConnection,nssmessages::SpillMessage> > p);

  void execute(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);

 private:
  boost::shared_ptr<gov::fnal::cd::rms::RmsSender<gov::fnal::cd::rms::provider::DDSConnection,
                    nssmessages::SpillMessage> >
        requestSenderptr;
};

class Dest
{
 public:
  Dest(std::string const& h,
       int const& p,
       std::list<int> const& tlist);

  std::string host;
  int port;
  std::list < int > typelist;
  XmlRpc::XmlRpcClient client;
};

class DestList //singleton, only one destination list per application
{
 public:
  static DestList& Instance();
  ~DestList() {;}

  std::vector< Dest > const& ReceiverList() {return recList;}

  int addDestination(std::string const & h,
		     int const & p,
		     std::list<int> const& tlist);

  void addAvailableSpillType(int const& t);
  std::list< int > const& AvailableSpillTypes() {return availableSpillTypes;}

 private:
  DestList();
  static DestList* fInstance;
  std::vector< Dest > recList;
  std::list< int > availableSpillTypes;

  void addList(std::vector<std::pair<std::basic_string<char>, int> > const& rL);
};

class SpillForward : public XmlRpc::XmlRpcServerMethod
{
 public:
  SpillForward(XmlRpc::XmlRpcServer* s);

  void execute(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);
  void DispatchTheSpill(XmlRpc::XmlRpcValue& params, int i );


 private:
};

class DestinationAdd : public XmlRpc::XmlRpcServerMethod
{
 public:
  DestinationAdd(XmlRpc::XmlRpcServer* s);

  void execute(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);

};

class DestinationRemove : public XmlRpc::XmlRpcServerMethod
{
 public:
  DestinationRemove(XmlRpc::XmlRpcServer* s);

  void execute(XmlRpc::XmlRpcValue& params, XmlRpc::XmlRpcValue& result);

};

} // end of namespace NovaSpillServer
} // end of namespace nova

#endif //__NSS_XMLRPCSERVERCLASSES_H_
