#ifndef SNDATABUFFERITERATOR__H
#define SNDATABUFFERITERATOR__H

#include "NovaSuperNova/SNDataBlock.h"

namespace supernova{
  // DataBuffer::iterator class is used for fast going forward through buffer
  /**
  * @file SNDataBufferIterator.h
  * @brief Iterator class for supernova::DataBuffer  
  * 
  * This class provides a fast way to iterate forward through all values in DataBuffer.
  * With this iterator you can use DataBuffer as any std::collection
  * Iterator allows to access not only stored value (@see value()), 
  * but also time of current bin (@see timestamp())
  * 
  * For usage example see unittest/testSNBuffer.cc
  *
  * @author Andrey Sheshukov
  * @version 1
  * @date 2015-04-17
  */
  class BufferIterator{
    public:
      BufferIterator(const std::deque<DataBlock>::reverse_iterator block_iter, size_t block_pos = 0):
          _blockIt(block_iter),_blockPos(block_pos)
            {   
            }

      //increment iterator - move to next position
      BufferIterator& operator ++(){
            ++_blockPos;
            if(_blockPos >= _blockIt->size()){
               ++_blockIt; _blockPos=0;
            }
            return *this;
          }
      bool operator !=(const BufferIterator& other){
         return (this->_blockIt!=other._blockIt) || (this->_blockPos!=other._blockPos);
      }
      //get stored value reference
      DataType & operator *() {return value(); }
      //get stored value reference
      DataType & value()      {return block().dataAtPos(_blockPos); }
      //get time of current point
      TimeType timestamp()    {return block().timeAtPos(_blockPos); }
      //get reference to current block
      DataBlock& block(){return (*_blockIt);}
      //get current position in block 
      size_t blockPos(){return _blockPos;}
    private:
      std::deque<DataBlock>::reverse_iterator _blockIt; /*< iterator to current block */
      size_t _blockPos; /*< position inside current block */
  };
}
#endif
