#include "NovaSuperNova/SNBase.h" //DataType and TimeType definitions

/**
* @file SNDataBlockInterface.h
* @brief Abstract interface for supernova::DataBlock supernova::DataBuffer
* @author Andrey Sheshukov
* @version 1
* @date 2015-04-17
*/

namespace supernova{
    /**
    * Class TimeRange
    * @brief a basic functionality for keeping [startTime, endTime] range 
    */
    class TimeRange{
      public:
        //ctor, which creates TimeRange in given range (regardless of order)
        TimeRange(const TimeType& timeStart, const TimeType& timeEnd):
          _time0(std::min(timeStart,timeEnd)),
          _time1(std::max(timeStart,timeEnd))
        {};
        //copy ctor
        TimeRange(const TimeRange &range):
          _time0(range.startTime()),
          _time1(range.endTime())
        {};

        ~TimeRange(){};
        
        //check if 
        bool   contains(const TimeType& t) const{return t>=startTime() && t<endTime();}
        TimeType duration() const {return endTime()-startTime();}
        const TimeType& startTime() const {return _time0;}
        const TimeType& endTime()  const {return _time1;}
      protected:
        TimeType _time0; /**< lower time limit*/
        TimeType _time1; /**< upper time limit*/
    };

    /**
    * @brief Virtual interface for Data storage buffer: 
    * access and fill data inside the storage, structured by timestamp
    */
    class VirtualDataBlock: public TimeRange{
      public:
      VirtualDataBlock(TimeRange range):TimeRange(range){}
      //data access by position operator
      DataType& operator[](size_t n){return dataAtPos(n);}
      //data access by position
      virtual DataType& dataAtPos(size_t n) = 0; 
      // check if buffer was never filled
      virtual bool empty() const{return true;}
      //maximal position
      virtual size_t size() const = 0;
      // fill value in timestamp posiotion
      virtual void fill(const TimeType& t, const DataType& d) = 0;
    };

}

