#include <messagefacility/MessageLogger/MessageLogger.h>

#include <NovaTimingUtilities/TimingUtilities.h>

#include <NovaGlobalTrigger/GTGlobals.h>
#include <NovaGlobalTrigger/GTLongTrigs.h>

#include "DAQDataFormats/RawTriggerTimingMarker.h"
#include "DAQDataFormats/RawTriggerRange.h"

using namespace std;
using namespace novadaq::gt;

// Takes a trigger longer than the millislice size, and produces a series of
// smaller triggers

/**
 * Constructor takes requested trigger length, and sets up list of triggers
 *
 */
GTLongTrigs::GTLongTrigs(uint32_t length) {
  NTrig = length / MilliSliceSize + 1;
  TrigCount = NTrig;
  TrigLeft = length % MilliSliceSize;
  first = true;
//  LOG_DEBUG("GTLongTrigs") << "A length " << length << " trigger makes " << (NTrig - 1)
//    << " triggers of millislice size " << MilliSliceSize
//    << " plus one more of length " << TrigLeft;
}


// Destructor not needed?
//GTLongTrigs::~GTLongTrigs() {
// nothing at present, we dynamically allocate no memory in here to free
//}


/**
 * Takes a fully-populated trigger element, corrects the time info
 *
 */
uint64_t GTLongTrigs::adjustTrigger(daqdataformats::RawTrigger *trigger)
{
// If this isn't the last trigger in the sequence, set length to
// millislice size, else use the leftover length
   daqdataformats::RawTriggerRange* triggerRange = trigger->getTriggerRange();
   if (TrigCount > 1)
     triggerRange->setTriggerLength(MilliSliceSize);
   else
     triggerRange->setTriggerLength(TrigLeft);

   if (first)
     first = false;
   else {
     // adjust the time window start to happen at the end of the old window
     // That's always MilliSliceSize forward in time, except for the
     // first one in the sequence, which already has the right time
     daqdataformats::RawTriggerTimingMarker* triggerTimingMarker = trigger->getTriggerTimingMarker();
      uint64_t trigTimeNova = triggerTimingMarker->getTriggerStart_baseClock();
     trigTimeNova += (MilliSliceSize * timeutils::NOVA_TIME_FACTOR / 2000000);
     triggerTimingMarker->setTriggerStart_baseClock(trigTimeNova);
     // pack our time marker into the trigger atom
     trigger->setTriggerTimingMarker(*triggerTimingMarker);
     // Special case - if the trigger we just adjusted has no length, 
     // return zero to stop the looping
     if (triggerRange->getTriggerLength() <= 0) return 0;
   }

// Otherwise, increment internal counter, return how many are left to do
  return TrigCount--;
}


// private methods.
