/* A thread that sends the triggers sitting on the queues */

#include <rms/provider/DDSConnection.h>
#include <rms/base/RmsDestination.h>
#include <rms/RmsSender.h>

#include <messagefacility/MessageLogger/MessageLogger.h>

#include <NovaGlobalTrigger/GT_SendQueues.h>

#include <DAQMessages/ccpp_GTMessages.h>
#include <NovaGlobalTrigger/GTGlobals.h>
#include <NovaGlobalTrigger/GTSendTrigAgent.h>

#include <boost/shared_ptr.hpp>

using namespace gov::fnal::cd::rms;
using namespace novadaq;
using namespace gt;

#define TRIGGER_APP_NAME "GTSendTrigAgent"

void GTSendTrigAgent() {

    mf::SetModuleName("GTTrigAgent");
    // the (singluar!) trigger we're working with: will get pointed to
    // the right place in memory during the config step
    // GTInfo *info;

    // Get the instance of the sending queues
    GT_SendQueues& theQueues = gt::GT_SendQueues::getInstance();

    // create the connection to RMS
    boost::shared_ptr<provider::DDSConnection>
        rmsConnection(new provider::DDSConnection(TRIGGER_APP_NAME,partNumber));

    // create the destination to represent where the messages will be sent
    base::RmsDestination sendDest(TRIGGER_MAILBOX,
                                  base::RmsDestination::TRIGGER_CHANNEL);

    // create a sender with the connection and destination
    RmsSender<provider::DDSConnection,
	      triggermessages::TriggerMessage>
              requestSender(rmsConnection, sendDest);

    // Loop over the send queues and send out the messages
    while (1) {
      
      // Iterate over the queues and get the next trigger message
      try{
	// LOG_DEBUG("GTSendTrigAgent") << "<======= Getting Next Trigger ======";
	triggermessages::TriggerMessage* theMessage = theQueues.GetNextTrigger();
	// Send the message
	requestSender.sendMessage(*theMessage);
	delete theMessage;
      }catch(...){
	::mf::LogWarning("GTSendTrigAgent") << "Polling for next trigger returned exception";
	// continue;
      }
      // LOG_DEBUG("GTSendTrigAgent") << "------- Spinning in Poll Looop -----" ;

    }

    // cleanup and exit
    requestSender.close();
    rmsConnection->close();
}
