#include <NovaGlobalTrigger/GT_InhibitQueue.h>

#include <sys/types.h>
#include <string>
#include <deque>

#include <boost/thread/mutex.hpp>


#include <messagefacility/MessageLogger/MessageLogger.h>

#include <DAQDataFormats/RawTriggerHeader.h>
#include <DAQDataFormats/RawTriggerTime.h>
#include <DAQDataFormats/RawTriggerTimingMarker.h>
#include <DAQDataFormats/RawTriggerRange.h>
#include <DAQDataFormats/RawTriggerMask.h>
#include <DAQDataFormats/RawTrigger.h>

// #include <NovaTimingUtilities/TimingUtilities.h>

#include <DAQMessages/ccpp_GTMessages.h>
#include <DAQMessages/ccpp_DDTMessages.h>
#include <NovaGlobalTrigger/GTGlobals.h>
#include <NovaGlobalTrigger/GTLongTrigs.h>

#include <NovaGlobalTrigger/GT.h>
#include <NovaGlobalTrigger/GTInfo.h>

// GT_NAMESPACE_BEGIN

inhibitQueue::inhibitQueue():messageQueue(){
}

bool inhibitQueue::checkInhibit(triggermessages::TriggerMessage trigIn){

  daqdataformats::RawTrigger theInhibitTrig;
  uint64_t inhibitStart;
  uint64_t inhibitEnd;

  // Extract the Trigger Object from the message
  daqdataformats::RawTrigger theTrigger;
  theTrigger.readData(trigIn.payload.get_buffer());

  // This is the base time of the trigger
  uint64_t triggerStart  = theTrigger.getTriggerTimingMarker()->getTriggerStart_baseClock();

  // The trigger stop time is then the deltaT shifted 5 bits and added to the base
  uint64_t triggerEnd = triggerStart + ( theTrigger.getTriggerRange()->getTriggerLength() << 5);

  // Check the inhibit queue for overlaps
  for(std::deque<triggermessages::TriggerMessage>::iterator it = theQueue.begin();
      it != theQueue.end(); 
      ++it){
    
    // Set the buffer for the trigger unpacking
    theInhibitTrig.readData(*it->payload.get_buffer());

    // Pull out the start and end times of the inhibit window
    inhibitStart = theInhibitTrig.getTriggerTimingMarker()->getTriggerStart_baseClock();
    inhibitEnd   = inhibitStart + ( theInhibitTrig.getTriggerRange()->getTriggerLength() << 5);

    // This is any overlap in the intervals
    if( (triggerStart < inhibitEnd) && (triggerEnd > inhibitStart) ){
      return true;
    }

    // This is the test trig is inside (fully) the interval
    if( (triggerStart < inhibitEnd   ) && 
	(triggerStart > inhibitStart ) &&
	(triggerEnd > inhibitStart   ) &&
	(triggerEnd < inhibitEnd     ) ){
      return true;
    }

    // This is the test trigger is in complete overlap with the interval
    if(	(triggerStart < inhibitStart ) &&
	(triggerEnd   > inhibitEnd   ) ){
      return true;
    }

    // This is the test trigger forward straddles the interval
    if(	(triggerStart < inhibitStart ) &&
	(triggerEnd   > inhibitStart ) &&
	(triggerEnd   < inhibitEnd   ) ){
      return true;
    }

    // This is the test trigger rear straddles the interval
    if(	(triggerStart > inhibitStart ) &&
	(triggerEnd   > inhibitStart ) &&
	(triggerEnd   > inhibitEnd   ) ){
      return true;
    }

  } // endfor it

  return false;
}

// GT_NAMESPACE_END
