#include <NovaGlobalTrigger/GT_MessageQueue.h>

#include <sys/types.h>
#include <string>
#include <deque>

#include <boost/thread/mutex.hpp>


#include <messagefacility/MessageLogger/MessageLogger.h>

// #include <DAQDataFormats/TriggerDefines.h>

// #include <NovaTimingUtilities/TimingUtilities.h>

#include <DAQMessages/ccpp_GTMessages.h>
#include <DAQMessages/ccpp_DDTMessages.h>
#include <NovaGlobalTrigger/GTGlobals.h>
#include <NovaGlobalTrigger/GTLongTrigs.h>


#include <NovaGlobalTrigger/GT.h>
#include <NovaGlobalTrigger/GTInfo.h>

// GT_NAMESPACE_BEGIN

messageQueue::messageQueue():max_queue_depth(-1),drop_count(0){
}

size_t messageQueue::size(){
    // Get the mutex
    boost::mutex::scoped_lock lock(theQueue_mutex);

    return theQueue.size();  
}

bool messageQueue::empty(){
  // Get the mutex
  boost::mutex::scoped_lock lock(theQueue_mutex);
    
    return theQueue.empty(); 
};

void messageQueue::flush(){
    // Get the mutex
    boost::mutex::scoped_lock lock(theQueue_mutex);
    drop_count += theQueue.size();
    theQueue.clear();
}

void messageQueue::reset(){
  boost::mutex::scoped_lock lock(theQueue_mutex);
  theQueue.clear();
  drop_count = 0;
}

int messageQueue::getDroppedCount(){
  boost::mutex::scoped_lock lock(theQueue_mutex);
  return drop_count;
}

bool messageQueue::queueMessage(triggermessages::TriggerMessage& newMessage){
  // Get the mutex
  boost::mutex::scoped_lock lock(theQueue_mutex);

  // If depth is set negative (unlimited)
  if(max_queue_depth < 0){
    // Push the message to the back of the queue
    theQueue.push_back(newMessage);
    return true;
  }
  
  // queue_depth is non-negative, so safe to cast to unsigned
  if((unsigned)max_queue_depth > theQueue.size()){
    // Push the message to the back of the queue
    theQueue.push_back(newMessage);      
    return true;
    }else{
    // Drop on input
    ++drop_count;
    return false;
  }
  
  return false;
}

triggermessages::TriggerMessage* messageQueue::popMessage(){
  // Get the mutex
  boost::mutex::scoped_lock lock(theQueue_mutex);
  
  // Retrieve the front of the queue
  triggermessages::TriggerMessage* theElement = new triggermessages::TriggerMessage(theQueue.front());
  
  // triggermessages::TriggerMessage& copyIn = theQueue.front();
  // Pop the top of the queue
  theQueue.pop_front();   
  
  // Return the element
  return theElement;
}

int messageQueue::setMaxQueueDepth(int value){
  boost::mutex::scoped_lock lock(theQueue_mutex);
  max_queue_depth = value;
  return max_queue_depth;
}

// GT_NAMESPACE_END
