#include "NovaTimingUtilities/TimingUtilities.h"
#include "TDUUtilities/tdu.h"
#include <iostream>
#include <string>
#include <ctime>
#include <cstdio>

using namespace novadaq::timeutils;

int main(int argc, char* argv[])
{
  std::cout << "Time on TDU:\n" << std::endl;

  uint64_t novatime;
  timeval now;
  gettimeofday(&now,0);
  std::cout << "The unix system time is " 
            << convertUnixTimeToString(now) << std::endl;

  convertUnixTimeToNovaTime(now, novatime);
  std::cout << "This is novatime " << novatime <<"\n   "
            << convertNovaTimeToString(novatime) <<"\n"<<std::endl;

  nova::TDUUtilities::tdu mtu; //Master Timing Unit or no GPS

  //Get control
  // First check the status register to make sure we aren't 
  // in the process of a command
  uint32_t status = mtu.getRegister(0x0001);
  while( status && 0x0004) {
   status = mtu.getRegister(0x0001);
   usleep(10);
  }
  
  // Now get the contents of the command register
  uint32_t ctrl = mtu.getRegister(0x0000);
  // To reload the current time you now need to tap bit 7 of the control
  // register (it used to be bit 5 which also did a sync
  ctrl |= (1<<7); //set bit 7, load nova time into reg. 0x10 to 0x13
  mtu.setRegister(0x0000,ctrl);
  //bit 5 should be 0 again after curtm registers are loaded by tdu

  // The time should now be calculated 
  // To make sure that it is we need to poll the status register
  // until it thinks it is safe to sync (meaning it calculated the time 

  status =0;
  status = mtu.getRegister(0x0001);
//  while ( (status && (~0x0008))){
//    status = mtu.getRegister(0x0001); 
//    usleep(10);
//  }

  uint32_t GPStowLo = 0; //Time of week low
  uint32_t GPStowHi = 0; //tow hi
  uint32_t GPSweek = 0;  //week
  GPStowLo = mtu.getRegister(0x0005);
  GPStowHi = mtu.getRegister(0x0006);
  GPSweek  = mtu.getRegister(0x0007); 
  
  std::cout <<"The MTU GPS week, time of week hi, and time of week lo are:\n"
            <<GPSweek<<" , "<<GPStowHi<<" , "<<GPStowLo<<"\n"<< std::endl;

  uint32_t curtm[4];
  uint64_t mtucurrent=0;
  for (int i=0; i!=4; ++i) {
    curtm[i] = mtu.getRegister(0x10+i);
    mtucurrent += ((uint64_t)curtm[i])<<(16*i);
  }
  printf( "Current GPS time: %04X %04X %04X %04X\n\n"
        , curtm[3], curtm[2], curtm[1], curtm[0] );

  uint64_t tdiff = mtucurrent-novatime;
  std::cout <<"This is decimal " << mtucurrent <<"\n"
            <<"Difference mtucurrent-unixcurrent: "
            <<tdiff <<" counts or "
            <<((double)tdiff)/((double)NOVA_TIME_FACTOR) <<" seconds.\n"
            <<"(This should be close to the number of leap seconds"
            <<" since the GPS epoch.)"<<std::endl;
  return 0;
}
